# -*- coding: utf-8 -*-
"""
	jikji/functions
	----------------
	global functions used in jikji or template

	:author: Prev(prevdev@gmail.com)
"""
from collections import OrderedDict
from .cprint import cprint



def group(array, *args) :
	""" Group json object by property
	
	:param array: json parsed array
	:param **kwarg: gruoping properties
	"""

	# for data in array :
	#print(array)

	result = OrderedDict()

	for data in array :
		keys = []
		for arg in args :
			cur = data # current value of data (go deeply)
			for property in arg :
				cur = cur[property]
			keys.append(cur) # find value in data by args

		cur = result
		for index, key in enumerate(keys) :
			# init cur[key] if not exists
			if key not in cur :
				if index == len(keys) - 1:	cur[key] = [] # init to array if key is lastest
				else :						cur[key] = OrderedDict()
			
			cur = cur[key] # go deeply

		cur.append(data)

	return result


# -*- coding: utf-8 -*-
"""
	Jikji/cli
	----------------
	Command line application for jikji app.
	
	:author: Prev(prevdev@gmail.com)
	
"""

import sys
import click

from .app import Jikji
from .cprint import cprint
from .listener import Listener


cli_help = """\
\b
Static website generator based on RESTFul Server *~*
You can read guide from https://github.com/Prev/jikji

\b 
Example Usage:
   $ jikji <mysite> generate
   $ jikji <mysite> cache list

"""



@click.group(help=cli_help)
@click.argument('sitepath', metavar='<sitepath>', type=click.Path(exists=True))
@click.pass_context
def cli(ctx, sitepath) :
	ctx.obj['SITEPATH'] = sitepath
	ctx.obj['APP'] = Jikji( sitepath )



"""
Command for generate
Usage:
	jikji <sitepath> generate

"""
@cli.command('generate', short_help="Generate static site")
@click.pass_context
def generate_command(ctx) :
	""" Generate static site
	"""
	app = ctx.obj['APP']
	app.generate()




"""
Command for cache
Usage:
	jikji <sitepath> cache <command> [--Args] [--Options]

"""
@cli.group('cache', help="Manage Cache")
def cache() :
	pass

@cache.command('list', short_help="Listing caches")
@click.pass_context
def cache_list_command(ctx) :
	app = ctx.obj['APP']

	for file in app.cache.list(details=True) :
		print(file)


@cache.command('clear', short_help="Remove all cache files")
@click.pass_context
def cache_clear_command(ctx) :
	app = ctx.obj['APP']
	app.cache.remove_all()

@cache.command('remove', short_help="Remove cache file matched")
@click.argument('key', metavar='<key>')
@click.option('--regex', '-r', is_flag=True, default=False, help="Using regex match instead of string equal")
@click.pass_context
def cache_remove_command(ctx, key, regex) :
	app = ctx.obj['APP']
	app.cache.remove(key, as_pattern=regex)




"""
Open listening server for develop
Usage:
	jikji <sitepath> listen

"""
@cli.command('listen')
@click.option('--host', '-h', default='0.0.0.0')
@click.option('--port', '-p', default=7000)
@click.pass_context
def listen_command(ctx, host, port) :
	""" Generate static site
	"""
	app = ctx.obj['APP']

	cprint.section('Rendering pages.xml')
	pages = app.generator.render_pages_xml( app.config.path.pages_xml )
	
	cprint.section('%s pages are opened' % len(pages))
	listener = Listener(app=app, pages=pages)

	cprint.section('Open Local Server with Flask')
	listener.listen(host=host, port=port)



def main(as_module=False) :
	""" Main function called from shell or __main__.py
	
	:param as_module: True if called from __main__
	"""
	name = __package__

	if as_module :
		name = 'python -m ' + name

	cli(prog_name = name, obj={})




# -*- coding: utf-8 -*-
"""
	Jikji
	----------------
	Static website generator based on RESTFul Server
	
	:author: Prev(prevdev@gmail.com)
	:license: MIT

"""

__version__ = '0.3.0'

from jikji.app import Jikji



# -*- coding: utf-8 -*-
"""
	jikji/config
	---------------
	Config Manage Class

	:author: Prev(prevdev@gmail.com)
"""

import os
import sys
import json
from .cprint import cprint


class ConfigPath :
	""" ConfigPath Object
		'path' property in config.json data
	"""

	_defaults = {
		'template': 'templates',
		'output': 'output',
		'assets': [],
		'pages_xml': 'pages.xml'
	}


	def __init__(self, sitepath, data) :
		""" Init ConfigPath Instance
		"""
		self.sitepath = sitepath
		self._data = data


	def _get(self, key) :
		""" Get data form _data with default value
		"""
		global _defaults
		return self._data.get(key, self._defaults[key]) # get value in dict with default value


	@property
	def tpl(self):
		return os.path.join(self.sitepath, self._get('template'))
	

	@property
	def output(self):
		return os.path.join(self.sitepath, self._get('output'))


	@property
	def assets(self):
		asset_lists = self._get('assets')
		return [ os.path.join(self.sitepath, d) for d in asset_lists ]


	@property
	def pages_xml(self):
		return os.path.join(self.sitepath, self._get('pages_xml'))



class Config :
	
	def __init__(self, config_file_path) :
		if os.path.isdir(config_file_path) :
			config_file_path += '/config.json'

		try :
			config_file = open(config_file_path, 'r')
			
		except FileNotFoundError :
			cprint.error('Error on jikji.Config.__init__')
			cprint.bold('Config file "%s" NOT FOUND' % config_file_path)
			sys.exit(-1)



		config_data = config_file.read()

		self._config = json.loads(config_data)
		self.sitepath = os.path.dirname(config_file_path)

		self._path_instance = ConfigPath(
			sitepath = self.sitepath,
			data = self._config.get('path', {})
		)


	@property
	def path(self):
		return self._path_instance
	
	
	@property
	def server_info(self) :
		return self._config.get('server_info', {})


	@property
	def log_history(self):
		return self._config.get('log_history', False)
	







# -*- coding: utf-8 -*-
"""
	jikji/model
	----------------
	Connect with Rest-Server

	:author: Prev(prevdev@gmail.com)
"""

import requests
import base64
import json
import os

from .cprint import cprint


class Model :

	__attrs__ =[
		'cache', '_baseurl', '_headers'	
	]


	def __init__(self, server_info, cache) :
		""" Init Model instance
		"""
		self.set_baseurl( server_info.get('base_url', '') )
		self.set_headers( server_info.get('headers', None) )
		self.cache = cache


	def set_baseurl(self, value) :
		if len(value) > 0 and value[-1] == '/' :
			value = value[0:-1]

		self._baseurl = value


	def get_baseurl(self) :
		return self._baseurl



	def set_headers(self, dictval) :
		if dictval is None :
			dictval = {}

		if dictval.get('Content-Type') is None :
			dictval['Content-Type'] = 'application/json'

		self._headers = dictval


	def get_headers(self) :
		return self._headers



	def geturl(self, api) :
		if api[0] != '/' :
			api = '/' + api
		return self.get_baseurl() + api



	def _rest(self, type, api, data=None, parsejson=True) :
		""" Rest Call to server
		:params
			- type: GET, POST, PUT, or DELETE
			- api: api string appended to baseurl
			- data: url form data (default: None)

		:return json-parsed object
		"""

		cprint.write("%s '%s' " % (type.upper(), api))

		if data is not None :
			data = json.dumps(data)


		# Call proper method of requests (maybe GET, POST or ext)
		r = getattr(requests, type.lower())(
			url = self.geturl(api),
			data = data,
			headers = self.get_headers()
		)

		try :
			r.raise_for_status()

		except requests.exceptions.HTTPError as e :
			# Raise ModelException when error occurs
			# Generater will handle Error and stop generating
			me = ModelException(HTTPError=e)
			cprint.error('%s' % me.status)
			raise me

		


		if parsejson :
			result = r.json()
		else :
			result = r.text
		
		cprint.ok('finish')
		return result


	def get(self, api, data=None, immutable=False) :
		""" Send a GET request to REST Sever
		:params
			- api: api string
			- data: url form data (default: None)
			- immutable: if immutable, make cache with key of api url (default: False)

		:return json-parsed object
		"""

		url = self.geturl(api)

		# If immutable, use cache
		if immutable == True :
			cachedata = self.cache.get(url)
			if cachedata is not None :
				cprint.write("GET '%s' " % api)
				cprint.ok('finish (use cache)')
				return cachedata


		result = self._rest('GET', api, data)

		if immutable == True :
			self.cache.set(url, result)

		return result



	def post(self, api, data=None) :
		""" Send a POST request to REST Sever
		:params
			- api: api string
			- data: url form data (default: None)

		:return json-parsed object
		"""
		return self._rest('POST', api, data)



	def put(self, api, data=None) :
		""" Send a PUT request to REST Sever
		:params
			- api: api string
			- data: url form data (default: None)

		:return json-parsed object
		"""
		return self._rest('PUT', api, data)



	def delete(self, api, data=None) :
		""" Send a DELETE request to REST Sever
		:params
			- api: api string
			- data: url form data (default: None)

		:return json-parsed object
		"""
		return self._rest('DELETE', api, data)





class ModelException(Exception) :

	def __init__(self, HTTPError) :
		code = HTTPError.response.status_code
		status_msg = requests.status_codes._codes[code][0].replace('_', ' ').upper()

		self.HTTPError = HTTPError
		self.status = '%s %s' % (code, status_msg)





# -*- coding: utf-8 -*-
"""
	jikji/utils
	----------------
	Utils of jikji
	History, Cache

	:author: Prev(prevdev@gmail.com)
"""

import os
import shutil
import json
import re
from datetime import datetime
from urllib.parse import quote_plus, unquote_plus

from .cprint import cprint


class AppDataUtil :
	def __init__(self, sitepath) :
		self.datapath = os.path.join(sitepath, '.jikji')



class Cache(AppDataUtil) :

	def __init__(self, sitepath) :
		""" Init Cache Class with sitepath
		"""

		AppDataUtil.__init__(self, sitepath)
		
		self.cachedir = os.path.join(self.datapath, 'cache')
		os.makedirs(self.cachedir, exist_ok=True )


	def getpath(self, key) :
		""" Get cache file path of key
		"""
		return os.path.join(self.cachedir, quote_plus(key))


	def get(self, key, default=None) :
		""" Get cached data with key
		If cache not found, return default value of param (default: None)
		"""
		cpath = self.getpath(key)
		
		if os.path.isfile(cpath) :
			with open(cpath, 'r') as file:
				content = file.read()

			return json.loads(content)

		else :
			return default


	def set(self, key, value) :
		""" Set cache data with key, value
		"""
		cpath = self.getpath(key)

		with open(cpath, 'w') as file:
			file.write( json.dumps(value) )


	def list(self, details=False) :
		""" Listing cache files
		"""
		clist = os.listdir(self.cachedir)

		if details :
			nlist = []

			for file in clist :
				filepath = os.path.join(self.cachedir, file)
				nlist.append('%s (%s bytes)' % (unquote_plus(file), os.path.getsize(filepath) ) )

			return nlist

		else :
			return clist


	def remove(self, key, as_pattern=False) :
		""" Remove cache data

		:param key: Key for cache or pattern if as_pattern is True
		:param as_pattern: Remove caches that matches with 'key' as regex
		"""

		if not as_pattern :
			cpath = self.getpath(key)

			if os.path.isfile(cpath) :
				os.remove(cpath)
				cprint.line('Cache "%s" is removed' % cpath)
			else :
				cprint.error('Cache "%s" not exists' % cpath)


		else :
			r = re.compile(key)
			clist = self.list()

			something_removed = False

			for file in clist :
				filepath = os.path.join(self.cachedir, file)

				if r.match(filepath) is not None :
					os.remove(filepath)

					something_removed = True
					cprint.line('Removed: %s' % filepath)

			if not something_removed :
				cprint.warn('No pattern matched file with "%s"' % key)


	def remove_all(self) :
		""" Remove all cache data
		"""
		self.remove('.*', as_pattern=True)




class History(AppDataUtil) :

	def __init__(self, config) :
		""" Init History Class with config
		"""

		AppDataUtil.__init__(self, config.sitepath)

		self._config = config
		if not self.log_enabled : return

		
		historydir = os.path.join(self.datapath, 'history')
		os.makedirs(historydir, exist_ok=True )


		# Get history dir of now time
		now = datetime.now()
		nowstr = now.strftime('%y-%m-%d %H:%M:%S.%f')
		cur_historydir = os.path.join(historydir, nowstr)


		# Make history dir of now time
		if not os.path.isdir(cur_historydir) :
			os.mkdir(cur_historydir)


		# Capture terminal log to "<cur_historydir>/terminal.log"
		cprint.capture_file( os.path.join(cur_historydir, 'terminal.log') )


		self.historydir = historydir
		self.cur_historydir = cur_historydir


	@property
	def log_enabled(self) :
		""" Return log_history is enabled
		"""
		return self._config.log_history



	def log(self, filename, content) :
		""" log file to cur_historydir
		"""
		if not self.log_enabled : return

		with open(os.path.join(self.cur_historydir, filename), 'w') as file:
			file.write(content)



	def log_generated_files(self) :
		""" log generated output files
		"""
		if not self.log_enabled : return
		
		try :
			shutil.copytree( self._config.path.output, os.path.join(self.cur_historydir, 'output') )
		except FileExistsError:
			pass


	def close(self) :
		""" Close opened files
		"""
		cprint.end_capture_file()






# -*- coding: utf-8 -*-
"""
	jikji/generator
	----------------
	Static page Generator

	First, read pages.xml and render this file via jinja2.
	while rendering pages.xml, context data will downloaded in rest-server.

	Next, parse rendered pages.xml and create static web pages.

	
	:author Prev(prevdev@gmail.com)
"""

import ast
import os
import sys
import time
import json
import jinja2
import xml.etree.ElementTree as ET
import traceback
import shutil

from .cprint import cprint
from .utils import History
from .model import ModelException
from . import functions

class Generator :
	
	__attrs__ =[
		'config', 'configpath', 'model', 'history'
		'_gen_start_time'
	]

	def __init__(self, config, model) :
		""" Constructor
		:param configpath: jikji.config.Config instance
		:param model: jikji.model.Model instance
		"""

		self.config = config
		self.configpath = config.path
		self.model = model
		self.history = None


	def generate(self) :
		""" Generate pages
			Read pages.xml, render with jinja2, parse rendered file, and create static web pages.
		"""

		self.history = History(self.config)
		self._gen_start_time = time.time()


		cprint.bold('Start generating "%s"\n' % os.path.abspath(self.configpath.sitepath))

		cprint.section('Parse pages.xml with Rest Server')
		cprint.bold('With Rest server "%s"\n' % self.model.get_baseurl())


		# render pages.xml
		try :
			pages = self.render_pages_xml( self.configpath.pages_xml )
		
		except ModelException as e:
			self._finish(False, 'Model Error', e)

		except jinja2.exceptions.TemplateError as e :
			self._finish(False, 'Template Error occurs in pages.xml', e)




		output_dir = self.configpath.output


		cprint.line()
		cprint.section('Rendering %d pages' % len(pages))
		cprint.bold('Output in "%s"\n' % output_dir)


		# template renderer Environment
		env = jinja2.Environment( loader=jinja2.FileSystemLoader( self.configpath.tpl ) )

		for page in pages :
			template = env.get_template( page['template'] )

			path = self._get_output_file_path(
				url = page['url'],
				output_dir = output_dir
			)

			# remove common string of output_dir in path
			trimed_path = path[ len(output_dir) : ]

			try :
				# Context is not parsed with json but ast
				#  because in pages xml, context value is not printed with json.dumps
				self._generate_page(
					output_file = path,
					context = page['context'],
					template = template,
				)
			
			except jinja2.exceptions.TemplateError as e :
				cprint.error('%s' % trimed_path )
				self._finish(False, 'Template Error', e)


			cprint.ok('%s' % trimed_path )


		for asset_dir in self.configpath.assets :
			self._copy_asset_files(
				asset_dir = asset_dir,
				output_dir = output_dir
			)


		self._finish(True)



	def _finish(self, is_success, err_cause=None, err_instance=None) :
		""" Called when generation is finished
			If some error occurs, stop generation and exit program
		
		:param is_success: boolean
		:param err_cause: If `is_success` is false, print cause of error
		:param err_instance: jinja2.exceptions.<SomeError> or ModelException
		"""

		if is_success :
			cprint.section('Generate completed in %s seconds' % round(time.time() - self._gen_start_time, 2), blue=True)
			
			self.history.log_generated_files()
			self.history.close()

		else :
			cprint.section()
			
			if type(err_instance) == jinja2.exceptions.TemplateSyntaxError :
				# Print template syntax error line and content of that line
				cprint.error( 'jinja2.exceptions.TemplateSyntaxError: ' + err_instance.message )
				cprint.line( '\nIn line %d:' % err_instance.lineno )
				cprint.line( err_instance.source.splitlines()[ err_instance.lineno - 1 ].strip() )
				cprint.line()

			else :
				traceback.print_exc()

			cprint.section('Generation Stopped by ' + err_cause , red=True)
			self.history.close()

			sys.exit(-1)

		

	def render_pages_xml(self, pages_xml_path) :
		""" Render pages.xml via jinja

		:param pages_xml_path: string
		:return array of pages data
		"""
		with open(pages_xml_path, 'r') as file:
			pages_config_content = file.read()

		
		tpl = jinja2.Template(pages_config_content)
		context = {
			'model': self.model,
			'time': time,
			'json': json
		}
		
		# Put functions to context
		for name, cls in functions.__dict__.items() :
			context[name] = cls


		rendered_xml = tpl.render(context)

		if self.history :
			self.history.log('pages.xml', rendered_xml)


		# parse rendered pages.xml via xml.etree.ElementTree
		page_tags = ET.fromstring(rendered_xml).findall('page')
		pages = []


		for page in page_tags :
			# parse context
			ctx_txt = page.find('context').text.strip()

			if page.find('context').attrib.get('type') == 'json' :
				ctx_dict = json.loads( ctx_txt )

			else :
				try :
					ctx_dict = ast.literal_eval( ctx_txt )
				except ValueError as e :
					cprint.warn(ctx_txt)
					self._finish(False, 'ValueError occurs while parse context in pages.xml', e)


			# append page data
			pages.append({
				'template': page.find('template').text.strip(),
				'url':		page.find('url').text.strip(),
				'context':	ctx_dict
			})

		return pages


	def _get_output_file_path(self, url, output_dir) :
		""" Get full path of output
		"""
		if url[-1] == '/' : url += 'index.html'
		if url[0] == '/' : url = url[1:]
		
		return output_dir + '/' + url



	def _generate_page(self, output_file, context, template) :
		""" Generate page via page_obj)

		:params
			- output_file: output_file_path(url + output_dir) (string)
			- context: context dict of template (string)
			- template: jinja2 template instance
		
		"""


		# render with jinja template
		output = template.render(context)

		# if dictionary not exists, make dirs
		os.makedirs( os.path.dirname(output_file), exist_ok=True )

		with open(output_file, 'w') as file:
			file.write(output)


	
	def _copy_asset_files(self, asset_dir, output_dir, dir=None) :
		""" Copy asset files in <asset_dir> declared in config.json
		
		:params
			- asset_dir: asset dir to copy
			- output_dir: output dir that copied file will located to
			- dir: dir path for recursively explored (if value is None, use asset_dir for first call)
		"""
		
		if dir is None : dir = asset_dir


		list = os.listdir(dir)
		
		for file in list :
			if file[0] == '.' :
				# continue if file is hidden
				continue

			filepath = os.path.join(dir, file)

			if os.path.isdir(filepath) :
				# if file is directory, call function recursively
				self._copy_asset_files(asset_dir, output_dir, filepath)

			else :
				# filepath that common string of asset_dir is removed
				trimed_path = filepath[ len(asset_dir)+1 : ] 
				dst_path = os.path.join(output_dir, trimed_path)

				os.makedirs( os.path.dirname(dst_path), exist_ok=True )
				shutil.copyfile(
					src = filepath,
					dst = dst_path
				)

				cprint.line('/%s [Asset]' % trimed_path)





# -*- coding: utf-8 -*-
"""
	jikji/app
	----------------
	Jikji class implements

	:author: Prev(prevdev@gmail.com)
"""

import os

from .config import Config
from .model import Model
from .generator import Generator
from .utils import Cache
from .cprint import cprint

from . import __version__

class Jikji :

	def __init__(self, config_path) :
		self._conf = Config( config_path )
		self._cache = Cache(self._conf.sitepath)

		self._model = Model(
			server_info = self._conf.server_info,
			cache = self._cache
		)

		self._generator = Generator(
			config = self._conf,
			model = self._model
		)

	@property
	def config(self) :
		return self._conf


	@property
	def model(self) :
		return self._model


	@property
	def cache(self) :
		return self._cache

	@property
	def generator(self) :
		return self._generator


	def generate(self) :
		""" Generate static website
		"""
		cprint.line('Using Jikji %s ' % __version__)
		
		self._generator.generate()





from setuptools import setup

import re
import ast

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('jikji/__init__.py', 'rb') as f:
	version = str(ast.literal_eval(_version_re.search(
		f.read().decode('utf-8')).group(1)))


setup(
	name = 'jikji',
	packages = ['jikji'],
	version = version,
	description = 'Static website generator based on RESTFul Server',
	license = 'MIT',

	author = 'Youngsoo Lee',
	author_email = 'prevdev@gmail.com',
	
	url = 'https://github.com/Prev/jikji',
	keywords = ['jikji', 'static', 'websites' 'generator', 'jinja2'],

	install_requires=[
		'Jinja2>=2.4',
		'click>=5.0',
		'Flask>=0.10',
		'requests>=2.11',
		'py>=1.4',
	],

	classifiers=(
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.3',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
	),

	entry_points={
		'console_scripts': [
			'jikji = jikji.cli:main',
		],
	},
)



# -*- coding: utf-8 -*-
"""
	jikji/listener
	----------------
	Listening server for fast develop

	:author: Prev(prevdev@gmail.com)
"""

import os
import mimetypes

import flask
import jinja2

class Listener :

	def __init__(self, app, pages) :
		""" Init Listener instance
		:param app: Jikji app istance
		:param pages: array of page info returned from generator:render_pages_xml
		"""
		self.app = app
		

		# change data format for pages
		# array to dict which key is url
		npages = {}
		for page in pages :
			url = self.format_url( page['url'] )
			npages[url] = ( page['template'], page['context'] )

			print('/' + url)

		self.pages = npages



	def listen(self, port, host) :
		""" Start listening HTTP server with Flask
		"""
		flaskapp = flask.Flask(__name__)

		flaskapp.add_url_rule('/', 'index', self.response)
		flaskapp.add_url_rule('/<path:url>', 'response', self.response)
		flaskapp.run(port=port, host=host)


	def format_url(self, url) :
		""" Format url to check equals
		"""
		if len(url) >= 10 and url[-10:] == 'index.html' :
			url = url[:-10]

		if len(url) >= 1 and url[0] == '/' :
			url = url[1:]
		if len(url) > 1 and url[-1] == '/' :
			url = url[0:-1]

		return url


	def response(self, url='') :
		""" Response content from url
			If url exists in pages.xml data, render template in realtime and return output
			Else, Find files in assets dir.
		"""
		url = self.format_url(url)

		if url in self.pages :
			tpl = self.pages[url][0]
			context = self.pages[url][1]

			# Render template with jinja
			env = jinja2.Environment( loader=jinja2.FileSystemLoader( self.app.config.path.tpl ) )
			template = env.get_template(tpl)

			output = template.render(context)
			return output


		for assetdir in self.app.config.path.assets :
			# Return asset if exists
			path = os.path.join(assetdir, url)

			if os.path.isfile(path) :
				with open(path, 'rb') as file :
					content = file.read()

				type = mimetypes.guess_type(url)[0]
				return content, 200, {'Content-type': type}


		return '<h1 align="center">404 NOT FOUND</h1>', 404
		





# -*- coding: utf-8 -*-

"""
	jikji/cprint
	------------------------------
	Print with color
	Capture terminal log to file
	
	:author Prev(prevdev@gmail.com)
"""

from py.io import TerminalWriter, StdCaptureFD

class cprint() :

	# Default Terminal Writer on 'py.std.sys.stdout'
	_tw = TerminalWriter()


	# Log Writer
	_lw = None
	_logfile = None


	def writers() :
		""" Return writers that be written
		"""
		if cprint._lw is not None :
			return (cprint._tw, cprint._lw)
		else :
			return (cprint._tw, )


	@staticmethod
	def capture_file(logfile_path) :
		""" Logging terminal output to logfile
		"""
		cprint._logfile = open(logfile_path, 'w+')
		cprint._lw = TerminalWriter(file=cprint._logfile) # log writer


	@staticmethod
	def end_capture_file() :
		""" End logging terminal output to logfile
		"""
		if cprint._logfile is not None :
			cprint._logfile.close()
		cprint._lw = None


	@staticmethod
	def ok(msg) :
		cprint.line(msg, green=True)


	@staticmethod
	def okb(msg) :
		cprint.line(msg, blue=True)


	@staticmethod
	def warn(msg) :
		cprint.line(msg, yellow=True)


	@staticmethod
	def fail(msg) :
		cprint.line(msg, red=True)
	

	@staticmethod
	def error(msg) :
		cprint.fail(msg)


	@staticmethod
	def bold(msg) :
		cprint.line(msg, bold=True)


	@staticmethod
	def write(msg, **markup) :
		for w in cprint.writers() :
			w.write(msg, **markup)
		

	@staticmethod
	def line(msg='', **markup) :
		for w in cprint.writers() :
			w.line(str(msg), **markup)


	@staticmethod
	def sep(sep, title, **markup) :
		for w in cprint.writers() :
			w.sep(sep, title, **markup)
		

	@staticmethod
	def section(title=None, **markup) :
		cprint.sep('-', title, **markup)


	